(ns poalses.jonathan.dialect.main
  "Dialect detector entrypoint"
  {:author "Jonathan Poalses"}
  (:require [clojure.string :as string]
            [clojure.tools.cli :as cli]
            [poalses.jonathan.dialect.dialect-nlp :as nlp]
            [taoensso.timbre :as log])
  (:gen-class))

(def ^:private log-levels
  "Supported log levels"
  #{:trace :debug :info :warn :error :fatal})

(def command-line-options
  "Command line options parsing rules."
   [["-l" "--log-level LEVEL" (str "Logging level " (seq log-levels))
     :default :info
     :parse-fn #(keyword (string/join (rest %)))
     :validate [#(contains? log-levels %)
                (str "Must be one of: " (seq log-levels))]]
    ["-d" "--development" "Development mode"
     :default false]
    ["-f" "--filename FILENAME" "Sets filename to be processed"
     :default "test.txt"]
    ["-h" "--help" "Displays usage"
     :default false]])

(defn- print-usage [options-summary]
  (println (->> ["Dialect Detector"
                 ""
                 "Usage: dialect-detector [options]"
                 ""
                 "Options:"
                 options-summary
                 ""
                 "Please refer to the manual page for more information."]
                (string/join \newline))))

(defn process-args
  "Processes command line `args`.
  If help was requested, prints a usage message and exits.
  If any problems were encountered parsing the command line, prints the errors and exits.

  Returns map of parsed options:

  | Key          | Value
  |--------------|------
  | :development | Boolean development mode flag
  | :log-level   | Desired logging level, one of (:trace :debug :info :warn :error :fatal)
  "
  [args]
  (let [{:keys [options errors summary]} (cli/parse-opts args command-line-options)]
    (cond
      (:help options)
      (do
        (print-usage summary)
        (System/exit 0))
      errors
      (do
        (println "Dialect Detector\n" (string/join \newline errors))
        (System/exit -1))
      :else options)))

(defn -main
  "Dialect Detector entrypoint."
  [& args]

  (let [options (process-args args)]
    (require '[taoensso.timbre :as log])
    (log/set-min-level! (:log-level options))
    (log/debug "Starting Dialect Detector...")
    (try
      (let [shutdown-trigger (promise)
            _bye-testing-hack (future (Thread/sleep 6000) (deliver shutdown-trigger true))]
        (log/info "Dialect Detector started up.")
        (nlp/detect-dialects (:filename options))
        @shutdown-trigger
        (log/info "Dialect Detector shutting down..."))
      (catch Exception e
        (log/fatal "Dialect Detector encountered a problem." e)
        (System/exit -666)))
    (log/info "Dialect Detector shut down.")
    (shutdown-agents)
    (System/exit 0)))

;;==============================================================================
;;TINKERING
;;==============================================================================
(comment

  (def test-vector [122 "blah blah blah" "more" 777])

  test-vector

  (count test-vector)

  (def test-list '(122 "blah blah blah" "more" 777))

  test-list

  (count test-list)

  (conj test-vector (first test-list))

  (conj test-vector (vec test-list))

  (take 21 (cycle test-vector))

  (into test-vector (vec (take 15 (repeatedly #(rand-int 10000)))))


  (def another-vector
    (delay [18746 "gjhsdfgh" 5857 58923]))

  another-vector

  @another-vector







  (into test-vector test-list)

  (conj test-vector 42)

  [test-list]

  (vec test-list)

  (conj test-list 42)

  *e)
;
; Dialect Detector.
;
; Copyright (c) 2023, Jonathan Poalses.
;
